@@ -31,12 +31,10 @@
 #include "response.h"
 #include "reason_phrase.h"
 
-#ifndef LINUX
-#ifndef MSG_NOSIGNAL
-#define MSG_NOSIGNAL 0
+#if HAVE_NETINET_TCP_H
+/* for TCP_CORK */
+#include <netinet/tcp.h>
 #endif
-#endif
-
 
 /**
  * Message to transmit when http 1.1 request is received
@@ -94,14 +92,6 @@
 #define INTERNAL_ERROR ""
 #endif
 
-#define EXTRA_CHECKS MHD_YES
-
-#if EXTRA_CHECKS
-#define EXTRA_CHECK(a) if (!(a)) abort();
-#else
-#define EXTRA_CHECK(a)
-#endif
-
 /**
  * Add extra debug messages with reasons for closing connections
  * (non-error reasons).
@@ -113,11 +103,6 @@
  */
 #define DEBUG_SEND_DATA MHD_NO
 
-/**
- * Should all state transitions be printed to stderr?
- */
-#define DEBUG_STATES MHD_NO
-
 /**
  * Get all of the headers from the request.
  *
@@ -153,6 +138,54 @@ MHD_get_connection_values (struct MHD_Connection *connection,
   return ret;
 }
 
+/**
+ * This function can be used to add an entry to
+ * the HTTP headers of a connection (so that the
+ * MHD_get_connection_values function will return
+ * them -- and the MHD PostProcessor will also
+ * see them).  This maybe required in certain
+ * situations (see Mantis #1399) where (broken)
+ * HTTP implementations fail to supply values needed
+ * by the post processor (or other parts of the
+ * application).
+ * <p>
+ * This function MUST only be called from within
+ * the MHD_AccessHandlerCallback (otherwise, access
+ * maybe improperly synchronized).  Furthermore,
+ * the client must guarantee that the key and
+ * value arguments are 0-terminated strings that
+ * are NOT freed until the connection is closed.
+ * (The easiest way to do this is by passing only
+ * arguments to permanently allocated strings.).
+ *
+ * @param connection the connection for which a
+ *  value should be set
+ * @param kind kind of the value
+ * @param key key for the value
+ * @param value the value itself
+ * @return MHD_NO if the operation could not be
+ *         performed due to insufficient memory;
+ *         MHD_YES on success
+ */
+int
+MHD_set_connection_value (struct MHD_Connection *connection,
+                          enum MHD_ValueKind kind,
+                          const char *key, const char *value)
+{
+  struct MHD_HTTP_Header *pos;
+
+  pos = MHD_pool_allocate (connection->pool,
+                           sizeof (struct MHD_HTTP_Header), MHD_NO);
+  if (pos == NULL)
+    return MHD_NO;
+  pos->header = (char *) key;
+  pos->value = (char *) value;
+  pos->kind = kind;
+  pos->next = connection->headers_received;
+  connection->headers_received = pos;
+  return MHD_YES;
+}
+
 /**
  * Get a particular header value.  If multiple
  * values match the kind, return any one of them.
@@ -180,7 +213,7 @@ MHD_lookup_connection_value (struct MHD_Connection *connection,
 
 /**
  * Queue a response to be transmitted to the client (as soon as
- * possible).
+ * possible but after MHD_AccessHandlerCallback returns).
  *
  * @param connection the connection identifying the client
  * @param status_code HTTP status code (i.e. 200 for OK)
@@ -208,7 +241,7 @@ MHD_queue_response (struct MHD_Connection *connection,
          have already sent the full message body */
       connection->response_write_position = response->total_size;
     }
-  if ((response->total_size == -1) &&
+  if ((response->total_size == MHD_SIZE_UNKNOWN) &&
       (0 == strcasecmp (connection->version, MHD_HTTP_VERSION_1_1)))
     connection->have_chunked_response = MHD_YES;
   else
@@ -246,6 +279,27 @@ need_100_continue (struct MHD_Connection *connection)
               strlen (HTTP_100_CONTINUE)));
 }
 
+/**
+ * Close the given connection and give the
+ * specified termination code to the user.
+ */
+void
+MHD_connection_close (struct MHD_Connection *connection,
+                      enum MHD_RequestTerminationCode termination_code)
+{
+  SHUTDOWN (connection->socket_fd, SHUT_RDWR);
+  CLOSE (connection->socket_fd);
+  connection->socket_fd = -1;
+  connection->state = MHD_CONNECTION_CLOSED;
+  if ( (NULL != connection->daemon->notify_completed) &&
+       (MHD_YES == connection->client_aware) )
+    connection->daemon->notify_completed (connection->daemon->
+					  notify_completed_cls, connection,
+					  &connection->client_context,
+					  termination_code);    
+  connection->client_aware = MHD_NO;
+}
+
 /**
  * A serious error occured, close the
  * connection (and notify the application).
@@ -253,15 +307,7 @@ need_100_continue (struct MHD_Connection *connection)
 static void
 connection_close_error (struct MHD_Connection *connection)
 {
-  SHUTDOWN (connection->socket_fd, SHUT_RDWR);
-  CLOSE (connection->socket_fd);
-  connection->socket_fd = -1;
-  connection->state = MHD_CONNECTION_CLOSED;
-  if (connection->daemon->notify_completed != NULL)
-    connection->daemon->notify_completed (connection->daemon->
-                                          notify_completed_cls, connection,
-                                          &connection->client_context,
-                                          MHD_REQUEST_TERMINATED_WITH_ERROR);
+  MHD_connection_close (connection, MHD_REQUEST_TERMINATED_WITH_ERROR);
 }
 
 /**
@@ -285,9 +331,12 @@ try_ready_normal_body (struct MHD_Connection *connection)
   ret = response->crc (response->crc_cls,
                        connection->response_write_position,
                        response->data,
-                       MIN (response->data_buffer_size,
-                            response->total_size -
-                            connection->response_write_position));
+                       MHD_MIN (response->data_buffer_size,
+                                response->total_size -
+                                connection->response_write_position));
+  if ((ret == 0) &&
+      (0 != (connection->daemon->options & MHD_USE_SELECT_INTERNALLY)))
+    abort ();                   /* serious client API violation */
   if (ret == -1)
     {
       /* either error or http 1.0 transfer, close
@@ -323,8 +372,9 @@ try_ready_chunked_body (struct MHD_Connection *connection)
   int ret;
   char *buf;
   struct MHD_Response *response;
-  unsigned int size;
-  char cbuf[9];
+  size_t size;
+  char cbuf[10];                /* 10: max strlen of "%x\r\n" */
+  int cblen;
 
   response = connection->response;
   if (connection->write_buffer_size == 0)
@@ -354,8 +404,8 @@ try_ready_chunked_body (struct MHD_Connection *connection)
 
   ret = response->crc (response->crc_cls,
                        connection->response_write_position,
-                       &connection->write_buffer[8],
-                       connection->write_buffer_size - 8 - 2);
+                       &connection->write_buffer[sizeof (cbuf)],
+                       connection->write_buffer_size - sizeof (cbuf) - 2);
   if (ret == -1)
     {
       /* end of message, signal other side! */
@@ -372,12 +422,14 @@ try_ready_chunked_body (struct MHD_Connection *connection)
     }
   if (ret > 0xFFFFFF)
     ret = 0xFFFFFF;
-  snprintf (cbuf, 8, "%X\r\n", ret);
-  memcpy (&connection->write_buffer[8 - strlen (cbuf)], cbuf, strlen (cbuf));
-  memcpy (&connection->write_buffer[8 + ret], "\r\n", 2);
+  SPRINTF (cbuf, "%X\r\n", ret);
+  cblen = strlen (cbuf);
+  EXTRA_CHECK (cblen <= sizeof (cbuf));
+  memcpy (&connection->write_buffer[sizeof (cbuf) - cblen], cbuf, cblen);
+  memcpy (&connection->write_buffer[sizeof (cbuf) + ret], "\r\n", 2);
   connection->response_write_position += ret;
-  connection->write_buffer_send_offset = 8 - strlen (cbuf);
-  connection->write_buffer_append_offset = 8 + ret + 2;
+  connection->write_buffer_send_offset = sizeof (cbuf) - cblen;
+  connection->write_buffer_append_offset = sizeof (cbuf) + ret + 2;
   return MHD_YES;
 }
 
@@ -392,7 +444,7 @@ add_extra_headers (struct MHD_Connection *connection)
   char buf[128];
 
   connection->have_chunked_upload = MHD_NO;
-  if (connection->response->total_size == -1)
+  if (connection->response->total_size == MHD_SIZE_UNKNOWN)
     {
       have = MHD_get_response_header (connection->response,
                                       MHD_HTTP_HEADER_CONNECTION);
@@ -419,10 +471,9 @@ add_extra_headers (struct MHD_Connection *connection)
   else if (NULL == MHD_get_response_header (connection->response,
                                             MHD_HTTP_HEADER_CONTENT_LENGTH))
     {
-      _REAL_SNPRINTF (buf,
-                      128,
-                      "%llu",
-                      (unsigned long long) connection->response->total_size);
+      SPRINTF (buf,
+               "%llu", 
+	       (unsigned long long)connection->response->total_size);
       MHD_add_response_header (connection->response,
                                MHD_HTTP_HEADER_CONTENT_LENGTH, buf);
     }
@@ -430,11 +481,12 @@ add_extra_headers (struct MHD_Connection *connection)
 
 /**
  * Produce HTTP "Date:" header.
- * @param date where to write the header
- * @param max maximum number of characters to write
+ *
+ * @param date where to write the header, with
+ *        at least 128 bytes available space.
  */
 static void
-get_date_string (char *date, unsigned int max)
+get_date_string (char *date)
 {
   static const char *days[] =
     { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
@@ -447,17 +499,17 @@ get_date_string (char *date, unsigned int max)
 
   time (&t);
   gmtime_r (&t, &now);
-  snprintf (date,
-            max - 1,
-            "Date: %3s, %02u %3s %04u %02u:%02u:%02u GMT\r\n",
-            days[now.tm_wday % 7],
-            now.tm_mday,
-            mons[now.tm_mon % 12],
-            1900 + now.tm_year, now.tm_hour, now.tm_min, now.tm_sec);
+  SPRINTF (date,
+           "Date: %3s, %02u %3s %04u %02u:%02u:%02u GMT\r\n",
+           days[now.tm_wday % 7],
+           now.tm_mday,
+           mons[now.tm_mon % 12],
+           1900 + now.tm_year, now.tm_hour, now.tm_min, now.tm_sec);
 }
 
 /**
- * try growing the read buffer
+ * Try growing the read buffer
+ *
  * @return MHD_YES on success, MHD_NO on failure
  */
 static int
@@ -491,7 +543,7 @@ build_header_response (struct MHD_Connection *connection)
   size_t size;
   size_t off;
   struct MHD_HTTP_Header *pos;
-  char code[128];
+  char code[256];
   char date[128];
   char *data;
   enum MHD_ValueKind kind;
@@ -501,15 +553,16 @@ build_header_response (struct MHD_Connection *connection)
     {
       add_extra_headers (connection);
       reason_phrase = MHD_get_reason_phrase_for (connection->responseCode);
-      _REAL_SNPRINTF (code, 128, "%s %u %s\r\n", MHD_HTTP_VERSION_1_1,
-                      connection->responseCode, reason_phrase);
+      SPRINTF (code,
+               "%s %u %s\r\n",
+               MHD_HTTP_VERSION_1_1, connection->responseCode, reason_phrase);
       off = strlen (code);
       /* estimate size */
       size = off + 2;           /* extra \r\n at the end */
       kind = MHD_HEADER_KIND;
       if (NULL == MHD_get_response_header (connection->response,
                                            MHD_HTTP_HEADER_DATE))
-        get_date_string (date, sizeof (date));
+        get_date_string (date);
       else
         date[0] = '\0';
       size += strlen (date);
@@ -544,10 +597,7 @@ build_header_response (struct MHD_Connection *connection)
   while (pos != NULL)
     {
       if (pos->kind == kind)
-        {
-          SPRINTF (&data[off], "%s: %s\r\n", pos->header, pos->value);
-          off += strlen (pos->header) + strlen (pos->value) + 4;
-        }
+        off += SPRINTF (&data[off], "%s: %s\r\n", pos->header, pos->value);
       pos = pos->next;
     }
   if (connection->state == MHD_CONNECTION_FOOTERS_RECEIVED)
@@ -555,7 +605,7 @@ build_header_response (struct MHD_Connection *connection)
       strcpy (&data[off], date);
       off += strlen (date);
     }
-  sprintf (&data[off], "\r\n");
+  memcpy (&data[off], "\r\n", 2);
   off += 2;
   if (off != size)
     abort ();
@@ -648,13 +698,17 @@ MHD_connection_get_fdset (struct MHD_Connection *connection,
   while (1)
     {
 #if DEBUG_STATES
-      fprintf (stderr, "`%s' in state %u\n", __FUNCTION__, connection->state);
+      MHD_DLOG (connection->daemon, "%s: state: %s\n",
+                __FUNCTION__, MHD_state_to_string (connection->state));
 #endif
       switch (connection->state)
         {
         case MHD_CONNECTION_INIT:
         case MHD_CONNECTION_URL_RECEIVED:
         case MHD_CONNECTION_HEADER_PART_RECEIVED:
+#if HTTPS_SUPPORT
+        case MHD_TLS_CONNECTION_INIT:
+#endif
           /* while reading headers, we always grow the
              read buffer if needed, no size-check required */
           if ((connection->read_closed) &&
@@ -764,6 +818,7 @@ MHD_connection_get_fdset (struct MHD_Connection *connection,
           if (connection->socket_fd != -1)
             connection_close_error (connection);
           return MHD_YES;       /* do nothing, not even reading */
+
         default:
           EXTRA_CHECK (0);
         }
@@ -902,8 +957,11 @@ parse_cookie_header (struct MHD_Connection *connection)
   const char *hdr;
   char *cpy;
   char *pos;
+  char *sce;
   char *semicolon;
   char *equals;
+  char *ekill;
+  char old;
   int quotes;
 
   hdr = MHD_lookup_connection_value (connection, MHD_HEADER_KIND, "Cookie");
@@ -923,11 +981,31 @@ parse_cookie_header (struct MHD_Connection *connection)
   pos = cpy;
   while (pos != NULL)
     {
-      equals = strstr (pos, "=");
-      if (equals == NULL)
-        break;
-      equals[0] = '\0';
-      equals++;
+      while (*pos == ' ')
+        pos++;                  /* skip spaces */
+
+      sce = pos;
+      while (((*sce) != '\0') &&
+             ((*sce) != ',') && ((*sce) != ';') && ((*sce) != '='))
+        sce++;
+      /* remove tailing whitespace (if any) from key */
+      ekill = sce - 1;
+      while ((*ekill == ' ') && (ekill >= pos))
+        *(ekill--) = '\0';
+      old = *sce;
+      *sce = '\0';
+      if (old != '=')
+        {
+          /* value part omitted, use empty string... */
+          if (MHD_NO ==
+              connection_add_header (connection, pos, "", MHD_COOKIE_KIND))
+            return MHD_NO;
+          if (old == '\0')
+            break;
+          pos = sce + 1;
+          continue;
+        }
+      equals = sce + 1;
       quotes = 0;
       semicolon = equals;
       while ((semicolon[0] != '\0') &&
@@ -987,6 +1065,11 @@ parse_initial_message_line (struct MHD_Connection *connection, char *line)
       httpVersion[0] = '\0';
       httpVersion++;
     }
+  if (connection->daemon->uri_log_callback != NULL)
+    connection->client_context
+      =
+      connection->daemon->uri_log_callback (connection->daemon->
+                                            uri_log_callback_cls, uri);
   args = strstr (uri, "?");
   if (args != NULL)
     {
@@ -994,6 +1077,7 @@ parse_initial_message_line (struct MHD_Connection *connection, char *line)
       args++;
       parse_arguments (MHD_GET_ARGUMENT_KIND, connection, args);
     }
+  MHD_http_unescape (uri);
   connection->url = uri;
   if (httpVersion == NULL)
     connection->version = "";
@@ -1010,21 +1094,24 @@ parse_initial_message_line (struct MHD_Connection *connection, char *line)
 static void
 call_connection_handler (struct MHD_Connection *connection)
 {
-  unsigned int processed;
-  unsigned int available;
-  unsigned int used;
+  size_t processed;
+  size_t available;
+  size_t used;
+  size_t i;
   int instant_retry;
-  unsigned int i;
   int malformed;
+  char *buffer_head;
 
   if (connection->response != NULL)
     return;                     /* already queued a response */
+
+  buffer_head = connection->read_buffer;
+  available = connection->read_buffer_offset;
   do
     {
       instant_retry = MHD_NO;
-      available = connection->read_buffer_offset;
       if ((connection->have_chunked_upload == MHD_YES) &&
-          (connection->remaining_upload_size == -1))
+          (connection->remaining_upload_size == MHD_SIZE_UNKNOWN))
         {
           if ((connection->current_chunk_offset ==
                connection->current_chunk_size)
@@ -1032,11 +1119,9 @@ call_connection_handler (struct MHD_Connection *connection)
             {
               /* skip new line at the *end* of a chunk */
               i = 0;
-              if ((connection->read_buffer[i] == '\r') ||
-                  (connection->read_buffer[i] == '\n'))
+              if ((buffer_head[i] == '\r') || (buffer_head[i] == '\n'))
                 i++;            /* skip 1st part of line feed */
-              if ((connection->read_buffer[i] == '\r') ||
-                  (connection->read_buffer[i] == '\n'))
+              if ((buffer_head[i] == '\r') || (buffer_head[i] == '\n'))
                 i++;            /* skip 2nd part of line feed */
               if (i == 0)
                 {
@@ -1048,10 +1133,8 @@ call_connection_handler (struct MHD_Connection *connection)
                   connection_close_error (connection);
                   return;
                 }
-              connection->read_buffer_offset -= i;
               available -= i;
-              memmove (connection->read_buffer,
-                       &connection->read_buffer[i], available);
+              buffer_head += i;
               connection->current_chunk_offset = 0;
               connection->current_chunk_size = 0;
             }
@@ -1066,8 +1149,7 @@ call_connection_handler (struct MHD_Connection *connection)
                 connection->current_chunk_offset;
               if (processed > available)
                 processed = available;
-              available -= processed;
-              if (available > 0)
+              if (available > processed)
                 instant_retry = MHD_YES;
             }
           else
@@ -1076,25 +1158,28 @@ call_connection_handler (struct MHD_Connection *connection)
               i = 0;
               while (i < available)
                 {
-                  if ((connection->read_buffer[i] == '\r') ||
-                      (connection->read_buffer[i] == '\n'))
+                  if ((buffer_head[i] == '\r') || (buffer_head[i] == '\n'))
                     break;
                   i++;
                   if (i >= 6)
                     break;
                 }
-              if (i >= available)
-                return;         /* need more data... */
+              /* take '\n' into account; if '\n'
+                 is the unavailable character, we
+                 will need to wait until we have it
+                 before going further */
+              if ((i + 1 >= available) &&
+                  !((i == 1) && (available == 2) && (buffer_head[0] == '0')))
+                break;          /* need more data... */
               malformed = (i >= 6);
               if (!malformed)
                 {
-                  connection->read_buffer[i] = '\0';
-                  malformed = (1 != sscanf (connection->read_buffer,
-                                            "%X",
-                                            &connection->current_chunk_size))
-                    && (1 !=
-                        sscanf (connection->read_buffer, "%x",
-                                &connection->current_chunk_size));
+                  buffer_head[i] = '\0';
+                  malformed =
+                    (1 != SSCANF (buffer_head, "%X",
+                                  &connection->current_chunk_size)) &&
+                    (1 != SSCANF (buffer_head, "%x",
+                                  &connection->current_chunk_size));
                 }
               if (malformed)
                 {
@@ -1107,18 +1192,20 @@ call_connection_handler (struct MHD_Connection *connection)
                   return;
                 }
               i++;
-              if ((connection->read_buffer[i] == '\r') ||
-                  (connection->read_buffer[i] == '\n'))
+              if ((i < available) &&
+                  ((buffer_head[i] == '\r') || (buffer_head[i] == '\n')))
                 i++;            /* skip 2nd part of line feed */
-              memmove (connection->read_buffer,
-                       &connection->read_buffer[i], available - i);
-              connection->read_buffer_offset -= i;
+
+              buffer_head += i;
+              available -= i;
               connection->current_chunk_offset = 0;
-              instant_retry = MHD_YES;
+
+              if (available > 0)
+                instant_retry = MHD_YES;
               if (connection->current_chunk_size == 0)
                 {
                   connection->remaining_upload_size = 0;
-                  return;
+                  break;
                 }
               continue;
             }
@@ -1127,17 +1214,16 @@ call_connection_handler (struct MHD_Connection *connection)
         {
           /* no chunked encoding, give all to the client */
           processed = available;
-          available = 0;
         }
       used = processed;
+      connection->client_aware = MHD_YES;
       if (MHD_NO ==
           connection->daemon->default_handler (connection->daemon->
                                                default_handler_cls,
                                                connection, connection->url,
                                                connection->method,
                                                connection->version,
-                                               connection->read_buffer,
-                                               &processed,
+                                               buffer_head, &processed,
                                                &connection->client_context))
         {
           /* serious internal error, close connection */
@@ -1156,14 +1242,15 @@ call_connection_handler (struct MHD_Connection *connection)
       if (connection->have_chunked_upload == MHD_YES)
         connection->current_chunk_offset += used;
       /* dh left "processed" bytes in buffer for next time... */
-      if (used > 0)
-        memmove (connection->read_buffer,
-                 &connection->read_buffer[used], processed + available);
-      if (connection->remaining_upload_size != -1)
+      buffer_head += used;
+      available -= used;
+      if (connection->remaining_upload_size != MHD_SIZE_UNKNOWN)
         connection->remaining_upload_size -= used;
-      connection->read_buffer_offset = processed + available;
     }
   while (instant_retry == MHD_YES);
+  if (available > 0)
+    memmove (connection->read_buffer, buffer_head, available);
+  connection->read_buffer_offset = available;
 }
 
 /**
@@ -1181,10 +1268,12 @@ do_read (struct MHD_Connection *connection)
 
   if (connection->read_buffer_size == connection->read_buffer_offset)
     return MHD_NO;
-  bytes_read = RECV (connection->socket_fd,
-                     &connection->read_buffer[connection->read_buffer_offset],
-                     connection->read_buffer_size -
-                     connection->read_buffer_offset, MSG_NOSIGNAL);
+
+  bytes_read = connection->recv_cls (connection,
+                                     &connection->read_buffer
+                                     [connection->read_buffer_offset],
+                                     connection->read_buffer_size -
+                                     connection->read_buffer_offset);
   if (bytes_read < 0)
     {
       if (errno == EINTR)
@@ -1207,6 +1296,67 @@ do_read (struct MHD_Connection *connection)
   return MHD_YES;
 }
 
+/**
+ * Try writing data to the socket from the
+ * write buffer of the connection.
+ *
+ * @return MHD_YES if something changed,
+ *         MHD_NO if we were interrupted
+ */
+static int
+do_write (struct MHD_Connection *connection)
+{
+  int ret;
+
+  ret = connection->send_cls (connection,
+                              &connection->write_buffer
+                              [connection->write_buffer_send_offset],
+                              connection->write_buffer_append_offset
+                              - connection->write_buffer_send_offset);
+
+  if (ret < 0)
+    {
+      if (errno == EINTR)
+        return MHD_NO;
+#if HAVE_MESSAGES
+      MHD_DLOG (connection->daemon,
+                "Failed to send data: %s\n", STRERROR (errno));
+#endif
+      connection_close_error (connection);
+      return MHD_YES;
+    }
+#if DEBUG_SEND_DATA
+  FPRINTF (stderr,
+           "Sent response: `%.*s'\n",
+           ret,
+           &connection->write_buffer[connection->write_buffer_send_offset]);
+#endif
+  connection->write_buffer_send_offset += ret;
+  return MHD_YES;
+}
+
+/**
+ * Check if we are done sending the write-buffer.
+ * If so, transition into "next_state".
+ * @return MHY_NO if we are not done, MHD_YES if we are
+ */
+static int
+check_write_done (struct MHD_Connection *connection,
+                  enum MHD_CONNECTION_STATE next_state)
+{
+  if (connection->write_buffer_append_offset !=
+      connection->write_buffer_send_offset)
+    return MHD_NO;
+  connection->write_buffer_append_offset = 0;
+  connection->write_buffer_send_offset = 0;
+  connection->state = next_state;
+  MHD_pool_reallocate (connection->pool, connection->write_buffer,
+                       connection->write_buffer_size, 0);
+  connection->write_buffer = NULL;
+  connection->write_buffer_size = 0;
+  return MHD_YES;
+}
+
 /**
  * We have received (possibly the beginning of) a line in the
  * header (or footer).  Validate (check for ":") and prepare
@@ -1345,7 +1495,7 @@ parse_connection_headers (struct MHD_Connection *connection)
                                       MHD_HTTP_HEADER_CONTENT_LENGTH);
   if (clen != NULL)
     {
-      if (1 != sscanf (clen, "%llu", &cval))
+      if (1 != SSCANF (clen, "%llu", &cval))
         {
 #if HAVE_MESSAGES
           MHD_DLOG (connection->daemon,
@@ -1368,7 +1518,7 @@ parse_connection_headers (struct MHD_Connection *connection)
         }
       else
         {
-          connection->remaining_upload_size = -1;       /* unknown size */
+          connection->remaining_upload_size = MHD_SIZE_UNKNOWN;
           if (0 ==
               strcasecmp (MHD_lookup_connection_value
                           (connection, MHD_HEADER_KIND,
@@ -1393,12 +1543,18 @@ MHD_connection_handle_read (struct MHD_Connection *connection)
   connection->last_activity = time (NULL);
   if (connection->state == MHD_CONNECTION_CLOSED)
     return MHD_NO;
+  /* make sure "read" has a reasonable number of bytes
+     in buffer to use per system call (if possible) */
+  if (connection->read_buffer_offset + MHD_BUF_INC_SIZE >
+      connection->read_buffer_size)
+    try_grow_read_buffer (connection);
   if (MHD_NO == do_read (connection))
     return MHD_YES;
   while (1)
     {
 #if DEBUG_STATES
-      fprintf (stderr, "`%s' in state %u\n", __FUNCTION__, connection->state);
+      MHD_DLOG (connection->daemon, "%s: state: %s\n",
+                __FUNCTION__, MHD_state_to_string (connection->state));
 #endif
       switch (connection->state)
         {
@@ -1435,67 +1591,6 @@ MHD_connection_handle_read (struct MHD_Connection *connection)
   return MHD_YES;
 }
 
-/**
- * Try writing data to the socket from the
- * write buffer of the connection.
- *
- * @return MHD_YES if something changed,
- *         MHD_NO if we were interrupted
- */
-static int
-do_write (struct MHD_Connection *connection)
-{
-  int ret;
-
-  ret = SEND (connection->socket_fd,
-              &connection->write_buffer[connection->
-                                        write_buffer_send_offset],
-              connection->write_buffer_append_offset -
-              connection->write_buffer_send_offset, MSG_NOSIGNAL);
-  if (ret < 0)
-    {
-      if (errno == EINTR)
-        return MHD_NO;
-#if HAVE_MESSAGES
-      MHD_DLOG (connection->daemon,
-                "Failed to send data: %s\n", STRERROR (errno));
-#endif
-      connection_close_error (connection);
-      return MHD_YES;
-    }
-#if DEBUG_SEND_DATA
-  fprintf (stderr,
-           "Sent HEADER response: `%.*s'\n",
-           ret,
-           &connection->write_buffer[connection->write_buffer_send_offset]);
-#endif
-  connection->write_buffer_send_offset += ret;
-  return MHD_YES;
-}
-
-/**
- * Check if we are done sending the write-buffer.
- * If so, transition into "next_state".
- * @return MHY_NO if we are not done, MHD_YES if we are
- */
-static int
-check_write_done (struct MHD_Connection *connection,
-                  enum MHD_CONNECTION_STATE next_state)
-{
-  if (connection->write_buffer_append_offset !=
-      connection->write_buffer_send_offset)
-    return MHD_NO;
-  connection->write_buffer_append_offset = 0;
-  connection->write_buffer_send_offset = 0;
-  connection->state = next_state;
-  MHD_pool_reallocate (connection->pool,
-                       connection->write_buffer,
-                       connection->write_buffer_size, 0);
-  connection->write_buffer = NULL;
-  connection->write_buffer_size = 0;
-  return MHD_YES;
-}
-
 /**
  * This function was created to handle writes to sockets when it has
  * been determined that the socket can be written to. All
@@ -1510,12 +1605,12 @@ MHD_connection_handle_write (struct MHD_Connection *connection)
 {
   struct MHD_Response *response;
   int ret;
-
   connection->last_activity = time (NULL);
   while (1)
     {
 #if DEBUG_STATES
-      fprintf (stderr, "`%s' in state %u\n", __FUNCTION__, connection->state);
+      MHD_DLOG (connection->daemon, "%s: state: %s\n",
+                __FUNCTION__, MHD_state_to_string (connection->state));
 #endif
       switch (connection->state)
         {
@@ -1528,12 +1623,11 @@ MHD_connection_handle_write (struct MHD_Connection *connection)
         case MHD_CONNECTION_HEADERS_PROCESSED:
           break;
         case MHD_CONNECTION_CONTINUE_SENDING:
-          ret = SEND (connection->socket_fd,
-                      &HTTP_100_CONTINUE[connection->
-                                         continue_message_write_offset],
-                      strlen (HTTP_100_CONTINUE) -
-                      connection->continue_message_write_offset,
-                      MSG_NOSIGNAL);
+          ret = connection->send_cls (connection,
+                                      &HTTP_100_CONTINUE
+                                      [connection->continue_message_write_offset],
+                                      strlen (HTTP_100_CONTINUE) -
+                                      connection->continue_message_write_offset);
           if (ret < 0)
             {
               if (errno == EINTR)
@@ -1546,11 +1640,11 @@ MHD_connection_handle_write (struct MHD_Connection *connection)
               return MHD_NO;
             }
 #if DEBUG_SEND_DATA
-          fprintf (stderr,
+          FPRINTF (stderr,
                    "Sent 100 continue response: `%.*s'\n",
                    ret,
-                   &HTTP_100_CONTINUE[connection->
-                                      continue_message_write_offset]);
+                   &HTTP_100_CONTINUE
+                   [connection->continue_message_write_offset]);
 #endif
           connection->continue_message_write_offset += ret;
           break;
@@ -1578,15 +1672,32 @@ MHD_connection_handle_write (struct MHD_Connection *connection)
               connection->state = MHD_CONNECTION_NORMAL_BODY_UNREADY;
               break;
             }
-          ret = SEND (connection->socket_fd,
-                      &response->data[connection->response_write_position -
-                                      response->data_start],
-                      response->data_size -
-                      (connection->response_write_position -
-                       response->data_start), MSG_NOSIGNAL);
+#if HTTPS_SUPPORT
+          if (connection->daemon->options & MHD_USE_SSL)
+            {
+              ret = MHD__gnutls_record_send (connection->tls_session,
+                                             &connection->response->data
+                                             [connection->
+                                              response_write_position -
+                                              response->data_start],
+                                             response->data_size -
+                                             (connection->response_write_position
+                                              - response->data_start));
+            }
+          else
+#endif
+            {
+              ret = connection->send_cls (connection,
+                                          &response->data
+                                          [connection->response_write_position
+                                           - response->data_start],
+                                          response->data_size -
+                                          (connection->response_write_position
+                                           - response->data_start));
+            }
 #if DEBUG_SEND_DATA
           if (ret > 0)
-            fprintf (stderr,
+            FPRINTF (stderr,
                      "Sent DATA response: `%.*s'\n",
                      ret,
                      &response->data[connection->response_write_position -
@@ -1617,8 +1728,7 @@ MHD_connection_handle_write (struct MHD_Connection *connection)
           do_write (connection);
           check_write_done (connection,
                             (connection->response->total_size ==
-                             connection->
-                             response_write_position) ?
+                             connection->response_write_position) ?
                             MHD_CONNECTION_BODY_SENT :
                             MHD_CONNECTION_CHUNKED_BODY_UNREADY);
           break;
@@ -1637,6 +1747,15 @@ MHD_connection_handle_write (struct MHD_Connection *connection)
           if (connection->socket_fd != -1)
             connection_close_error (connection);
           return MHD_NO;
+        case MHD_TLS_CONNECTION_INIT:
+        case MHD_TLS_HELLO_REQUEST:
+        case MHD_TLS_HANDSHAKE_FAILED:
+          EXTRA_CHECK (0);
+          break;
+        default:
+          EXTRA_CHECK (0);
+          connection_close_error (connection);
+          return MHD_NO;
         }
       break;
     }
@@ -1662,7 +1781,8 @@ MHD_connection_handle_idle (struct MHD_Connection *connection)
   while (1)
     {
 #if DEBUG_STATES
-      fprintf (stderr, "`%s' in state %u\n", __FUNCTION__, connection->state);
+      MHD_DLOG (connection->daemon, "%s: state: %s\n",
+                __FUNCTION__, MHD_state_to_string (connection->state));
 #endif
       switch (connection->state)
         {
@@ -1748,6 +1868,13 @@ MHD_connection_handle_idle (struct MHD_Connection *connection)
               connection->state = MHD_CONNECTION_CONTINUE_SENDING;
               break;
             }
+          if (connection->response != NULL)
+            {
+              /* we refused (no upload allowed!) */
+              connection->remaining_upload_size = 0;
+              /* force close, in case client still tries to upload... */
+              connection->read_closed = MHD_YES;
+            }
           connection->state = (connection->remaining_upload_size == 0)
             ? MHD_CONNECTION_FOOTERS_RECEIVED : MHD_CONNECTION_CONTINUE_SENT;
           continue;
@@ -1767,7 +1894,7 @@ MHD_connection_handle_idle (struct MHD_Connection *connection)
                 continue;
             }
           if ((connection->remaining_upload_size == 0) ||
-              ((connection->remaining_upload_size == -1) &&
+              ((connection->remaining_upload_size == MHD_SIZE_UNKNOWN) &&
                (connection->read_buffer_offset == 0) &&
                (MHD_YES == connection->read_closed)))
             {
@@ -1845,6 +1972,15 @@ MHD_connection_handle_idle (struct MHD_Connection *connection)
               continue;
             }
           connection->state = MHD_CONNECTION_HEADERS_SENDING;
+
+#if HAVE_DECL_TCP_CORK
+          /* starting header send, set TCP cork */
+          {
+            const int val = 1;
+            setsockopt (connection->socket_fd, IPPROTO_TCP, TCP_CORK, &val,
+                        sizeof (val));
+          }
+#endif
           break;
         case MHD_CONNECTION_HEADERS_SENDING:
           /* no default action */
@@ -1900,16 +2036,27 @@ MHD_connection_handle_idle (struct MHD_Connection *connection)
           /* no default action */
           break;
         case MHD_CONNECTION_FOOTERS_SENT:
+#if HAVE_DECL_TCP_CORK
+          /* done sending, uncork */
+          {
+            const int val = 0;
+            setsockopt (connection->socket_fd, IPPROTO_TCP, TCP_CORK, &val,
+                        sizeof (val));
+          }
+#endif
           MHD_destroy_response (connection->response);
           if (connection->daemon->notify_completed != NULL)
-            connection->daemon->notify_completed (connection->daemon->
-                                                  notify_completed_cls,
-                                                  connection,
-                                                  &connection->client_context,
-                                                  MHD_REQUEST_TERMINATED_COMPLETED_OK);
-          end = MHD_lookup_connection_value (connection,
-                                             MHD_HEADER_KIND,
-                                             MHD_HTTP_HEADER_CONNECTION);
+	    {
+	      connection->daemon->notify_completed (connection->daemon->
+						    notify_completed_cls,
+						    connection,
+						    &connection->client_context,
+						    MHD_REQUEST_TERMINATED_COMPLETED_OK);
+	    }
+	  connection->client_aware = MHD_NO;
+          end =
+            MHD_lookup_connection_value (connection, MHD_HEADER_KIND,
+                                         MHD_HTTP_HEADER_CONNECTION);
           connection->client_context = NULL;
           connection->continue_message_write_offset = 0;
           connection->responseCode = 0;
@@ -1965,11 +2112,59 @@ MHD_connection_handle_idle (struct MHD_Connection *connection)
   if ((connection->socket_fd != -1) &&
       (timeout != 0) && (time (NULL) - timeout > connection->last_activity))
     {
-      connection_close_error (connection);
+      MHD_connection_close (connection, MHD_REQUEST_TERMINATED_TIMEOUT_REACHED);
       return MHD_NO;
     }
   return MHD_YES;
 
 }
 
+void
+MHD_set_http_calbacks (struct MHD_Connection *connection)
+{
+  connection->read_handler = &MHD_connection_handle_read;
+  connection->write_handler = &MHD_connection_handle_write;
+  connection->idle_handler = &MHD_connection_handle_idle;
+}
+
+#if HTTPS_SUPPORT
+#include "gnutls_int.h"
+#include "gnutls_record.h"
+#endif
+
+/**
+ * Obtain information about the given connection.
+ *
+ * @param connection what connection to get information about
+ * @param infoType what information is desired?
+ * @param ... depends on infoType
+ * @return NULL if this information is not available
+ *         (or if the infoType is unknown)
+ */
+const union MHD_ConnectionInfo *
+MHD_get_connection_info (struct MHD_Connection *connection,
+                         enum MHD_ConnectionInfoType infoType, ...)
+{
+  switch (infoType)
+    {
+#if HTTPS_SUPPORT
+    case MHD_CONNECTION_INFO_CIPHER_ALGO:
+      if (connection->tls_session == NULL)
+	return NULL;
+      return (const union MHD_ConnectionInfo *) &connection->
+        tls_session->security_parameters.read_bulk_cipher_algorithm;
+    case MHD_CONNECTION_INFO_PROTOCOL:
+      if (connection->tls_session == NULL)
+	return NULL;
+      return (const union MHD_ConnectionInfo *) &connection->
+        tls_session->security_parameters.version;
+#endif
+    case MHD_CONNECTION_INFO_CLIENT_ADDRESS:
+      return (const union MHD_ConnectionInfo *) &connection->addr;
+    default:
+      return NULL;
+    };
+}
+
+
 /* end of connection.c */
