@@ -1,6 +1,6 @@
 /*
      This file is part of libmicrohttpd
-     (C) 2007 Daniel Pittman and Christian Grothoff
+     (C) 2007, 2009 Daniel Pittman and Christian Grothoff
 
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Lesser General Public
@@ -22,13 +22,16 @@
  * @brief memory pool
  * @author Christian Grothoff
  */
-
 #include "memorypool.h"
 
 /* define MAP_ANONYMOUS for Mac OS X */
 #if defined(MAP_ANON) && !defined(MAP_ANONYMOUS)
 #define MAP_ANONYMOUS MAP_ANON
 #endif
+#ifndef MAP_FAILED
+#define MAP_FAILED ((void*)-1)
+#endif
+
 
 struct MemoryPool
 {
@@ -41,17 +44,17 @@ struct MemoryPool
   /**
    * Size of the pool.
    */
-  unsigned int size;
+  size_t size;
 
   /**
    * Offset of the first unallocated byte.
    */
-  unsigned int pos;
+  size_t pos;
 
   /**
    * Offset of the last unallocated byte.
    */
-  unsigned int end;
+  size_t end;
 
   /**
    * MHD_NO if pool was malloc'ed, MHD_YES if mmapped.
@@ -65,15 +68,19 @@ struct MemoryPool
  * @param max maximum size of the pool
  */
 struct MemoryPool *
-MHD_pool_create (unsigned int max)
+MHD_pool_create (size_t max)
 {
   struct MemoryPool *pool;
 
   pool = malloc (sizeof (struct MemoryPool));
   if (pool == NULL)
     return NULL;
+#ifdef MAP_ANONYMOUS
   pool->memory = MMAP (NULL, max, PROT_READ | PROT_WRITE,
                        MAP_ANONYMOUS, -1, 0);
+#else
+  pool->memory = MAP_FAILED;
+#endif
   if ((pool->memory == MAP_FAILED) || (pool->memory == NULL))
     {
       pool->memory = malloc (max);
@@ -115,7 +122,8 @@ MHD_pool_destroy (struct MemoryPool *pool)
  *         bytes
  */
 void *
-MHD_pool_allocate (struct MemoryPool *pool, unsigned int size, int from_end)
+MHD_pool_allocate (struct MemoryPool *pool, 
+		   size_t size, int from_end)
 {
   void *ret;
 
@@ -152,7 +160,9 @@ MHD_pool_allocate (struct MemoryPool *pool, unsigned int size, int from_end)
  */
 void *
 MHD_pool_reallocate (struct MemoryPool *pool,
-                     void *old, unsigned int old_size, unsigned int new_size)
+                     void *old, 
+		     size_t old_size, 
+		     size_t new_size)
 {
   void *ret;
 
@@ -197,7 +207,9 @@ MHD_pool_reallocate (struct MemoryPool *pool,
  * @return addr new address of "keep" (if it had to change)
  */
 void *
-MHD_pool_reset (struct MemoryPool *pool, void *keep, unsigned int size)
+MHD_pool_reset (struct MemoryPool *pool, 
+		void *keep, 
+		size_t size)
 {
   if (keep != NULL)
     {
