@@ -1,6 +1,6 @@
 /*
      This file is part of libmicrohttpd
-     (C) 2007 Daniel Pittman and Christian Grothoff
+     (C) 2007, 2009 Daniel Pittman and Christian Grothoff
 
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Lesser General Public
@@ -172,22 +172,22 @@ struct MHD_PostProcessor
   /**
    * Size of our buffer for the key.
    */
-  unsigned int buffer_size;
+  size_t buffer_size;
 
   /**
    * Current position in the key buffer.
    */
-  unsigned int buffer_pos;
+  size_t buffer_pos;
 
   /**
    * Current position in xbuf.
    */
-  unsigned int xbuf_pos;
+  size_t xbuf_pos;
 
   /**
    * Current offset in the value being processed.
    */
-  unsigned int value_offset;
+  uint64_t value_offset;
 
   /**
    * strlen(boundary) -- if boundary != NULL.
@@ -247,7 +247,7 @@ struct MHD_PostProcessor
  */
 struct MHD_PostProcessor *
 MHD_create_post_processor (struct MHD_Connection *connection,
-                           unsigned int buffer_size,
+                           size_t buffer_size,
                            MHD_PostDataIterator ikvi, void *cls)
 {
   struct MHD_PostProcessor *ret;
@@ -263,7 +263,8 @@ MHD_create_post_processor (struct MHD_Connection *connection,
   if (encoding == NULL)
     return NULL;
   boundary = NULL;
-  if (0 != strcasecmp (MHD_HTTP_POST_ENCODING_FORM_URLENCODED, encoding))
+  if (0 != strncasecmp (MHD_HTTP_POST_ENCODING_FORM_URLENCODED, encoding,
+                        strlen (MHD_HTTP_POST_ENCODING_MULTIPART_FORMDATA)))
     {
       if (0 !=
           strncasecmp (MHD_HTTP_POST_ENCODING_MULTIPART_FORMDATA, encoding,
@@ -303,13 +304,14 @@ MHD_create_post_processor (struct MHD_Connection *connection,
  */
 static int
 post_process_urlencoded (struct MHD_PostProcessor *pp,
-                         const char *post_data, unsigned int post_data_len)
+                         const char *post_data, 
+			 size_t post_data_len)
 {
-  unsigned int equals;
-  unsigned int amper;
-  unsigned int poff;
-  unsigned int xoff;
-  unsigned int delta;
+  size_t equals;
+  size_t amper;
+  size_t poff;
+  size_t xoff;
+  size_t delta;
   int end_of_value_found;
   char *buf;
   char xbuf[XBUF_SIZE + 1];
@@ -356,15 +358,15 @@ post_process_urlencoded (struct MHD_PostProcessor *pp,
           /* find last position in input buffer that is part of the value */
           amper = 0;
           while ((amper + poff < post_data_len) &&
-		 (amper < XBUF_SIZE) && 
+                 (amper < XBUF_SIZE) &&
                  (post_data[amper + poff] != '&') &&
                  (post_data[amper + poff] != '\n') &&
                  (post_data[amper + poff] != '\r'))
             amper++;
-	  end_of_value_found = ( (amper + poff < post_data_len) &&
-				 ( (post_data[amper + poff] == '&') ||
-				   (post_data[amper + poff] == '\n') ||
-				   (post_data[amper + poff] == '\r') ) );
+          end_of_value_found = ((amper + poff < post_data_len) &&
+                                ((post_data[amper + poff] == '&') ||
+                                 (post_data[amper + poff] == '\n') ||
+                                 (post_data[amper + poff] == '\r')));
           /* compute delta, the maximum number of bytes that we will be able to
              process right now (either amper-limited of xbuf-size limited) */
           delta = amper;
@@ -402,32 +404,30 @@ post_process_urlencoded (struct MHD_PostProcessor *pp,
 
           /* unescape */
           xbuf[xoff] = '\0';    /* 0-terminate in preparation */
-          MHD_http_unescape (xbuf);
-
+          xoff = MHD_http_unescape (xbuf);
           /* finally: call application! */
-          if (MHD_NO ==
-	      pp->ikvi (pp->cls, MHD_POSTDATA_KIND, (const char *) &pp[1],  /* key */
-			NULL, NULL, NULL, xbuf, pp->value_offset, xoff))
-	    {
-	      pp->state = PP_Error;
-	      return MHD_NO;
-	    }
+          if (MHD_NO == pp->ikvi (pp->cls, MHD_POSTDATA_KIND, (const char *) &pp[1],    /* key */
+                                  NULL, NULL, NULL, xbuf, pp->value_offset,
+                                  xoff))
+            {
+              pp->state = PP_Error;
+              return MHD_NO;
+            }
           pp->value_offset += xoff;
 
           /* are we done with the value? */
           if (end_of_value_found)
             {
               /* we found the end of the value! */
-              if ((post_data[poff] == '\n') ||
-                  (post_data[poff] == '\r'))
-		{
-		  pp->state = PP_ExpectNewLine;
-		}
-	      else
-		{
-		  poff++;           /* skip '&' */
-		  pp->state = PP_Init;
-		}
+              if ((post_data[poff] == '\n') || (post_data[poff] == '\r'))
+                {
+                  pp->state = PP_ExpectNewLine;
+                }
+              else
+                {
+                  poff++;       /* skip '&' */
+                  pp->state = PP_Init;
+                }
             }
           break;
         case PP_ExpectNewLine:
@@ -473,7 +473,7 @@ static int
 find_boundary (struct MHD_PostProcessor *pp,
                const char *boundary,
                size_t blen,
-               unsigned int *ioffptr,
+               size_t *ioffptr,
                enum PP_State next_state, enum PP_State next_dash_state)
 {
   char *buf = (char *) &pp[1];
@@ -555,10 +555,10 @@ try_get_value (const char *buf, const char *key, char **destination)
  */
 static int
 process_multipart_headers (struct MHD_PostProcessor *pp,
-                           unsigned int *ioffptr, enum PP_State next_state)
+                           size_t *ioffptr, enum PP_State next_state)
 {
   char *buf = (char *) &pp[1];
-  unsigned int newline;
+  size_t newline;
 
   newline = 0;
   while ((newline < pp->buffer_pos) &&
@@ -616,14 +616,14 @@ process_multipart_headers (struct MHD_PostProcessor *pp,
  */
 static int
 process_value_to_boundary (struct MHD_PostProcessor *pp,
-                           unsigned int *ioffptr,
+                           size_t *ioffptr,
                            const char *boundary,
                            size_t blen,
                            enum PP_State next_state,
                            enum PP_State next_dash_state)
 {
   char *buf = (char *) &pp[1];
-  unsigned int newline;
+  size_t newline;
 
   /* all data in buf until the boundary
      (\r\n--+boundary) is part of the value */
@@ -717,12 +717,13 @@ free_unmarked (struct MHD_PostProcessor *pp)
  */
 static int
 post_process_multipart (struct MHD_PostProcessor *pp,
-                        const char *post_data, unsigned int post_data_len)
+                        const char *post_data,
+			size_t post_data_len)
 {
   char *buf;
-  unsigned int max;
-  unsigned int ioff;
-  unsigned int poff;
+  size_t max;
+  size_t ioff;
+  size_t poff;
   int state_changed;
 
   buf = (char *) &pp[1];
@@ -1000,13 +1001,14 @@ END:
  */
 int
 MHD_post_process (struct MHD_PostProcessor *pp,
-                  const char *post_data, unsigned int post_data_len)
+                  const char *post_data, size_t post_data_len)
 {
   if (post_data_len == 0)
     return MHD_YES;
   if (pp == NULL)
     return MHD_NO;
-  if (0 == strcasecmp (MHD_HTTP_POST_ENCODING_FORM_URLENCODED, pp->encoding))
+  if (0 == strncasecmp (MHD_HTTP_POST_ENCODING_FORM_URLENCODED, pp->encoding,
+                         strlen(MHD_HTTP_POST_ENCODING_FORM_URLENCODED)))
     return post_process_urlencoded (pp, post_data, post_data_len);
   if (0 ==
       strncasecmp (MHD_HTTP_POST_ENCODING_MULTIPART_FORMDATA, pp->encoding,
@@ -1027,8 +1029,7 @@ MHD_destroy_post_processor (struct MHD_PostProcessor *pp)
   /* These internal strings need cleaning up since
      the post-processing may have been interrupted
      at any stage */
-  if ( (pp->xbuf_pos > 0) ||
-       (pp->state != PP_Done) )
+  if ((pp->xbuf_pos > 0) || (pp->state != PP_Done))
     ret = MHD_NO;
   else
     ret = MHD_YES;
