@@ -1,21 +1,20 @@
 /*
      This file is part of libmicrohttpd
-     (C) 2007 Christian Grothoff
+     (C) 2007 Christian Grothoff (and other contributing authors)
 
-     libmicrohttpd is free software; you can redistribute it and/or modify
-     it under the terms of the GNU General Public License as published
-     by the Free Software Foundation; either version 2, or (at your
-     option) any later version.
+     This library is free software; you can redistribute it and/or
+     modify it under the terms of the GNU Lesser General Public
+     License as published by the Free Software Foundation; either
+     version 2.1 of the License, or (at your option) any later version.
 
-     libmicrohttpd is distributed in the hope that it will be useful, but
-     WITHOUT ANY WARRANTY; without even the implied warranty of
+     This library is distributed in the hope that it will be useful,
+     but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-     General Public License for more details.
+     Lesser General Public License for more details.
 
-     You should have received a copy of the GNU General Public License
-     along with libmicrohttpd; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     You should have received a copy of the GNU Lesser General Public
+     License along with this library; if not, write to the Free Software
+     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 /**
@@ -24,21 +23,14 @@
  * @author Christian Grothoff
  */
 
-#include "config.h"
+#include "platform.h"
 #include <microhttpd.h>
-#include <stdlib.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#ifndef MINGW
 #include <unistd.h>
-#endif
-#include <string.h>
-#include <stdio.h>
 
 #define PAGE "<html><head><title>File not found</title></head><body>File not found</body></html>"
 
 static int
-file_reader (void *cls, size_t pos, char *buf, int max)
+file_reader (void *cls, uint64_t pos, char *buf, int max)
 {
   FILE *file = cls;
 
@@ -51,8 +43,9 @@ ahc_echo (void *cls,
           struct MHD_Connection *connection,
           const char *url,
           const char *method,
+          const char *version,
           const char *upload_data,
-          const char *version, unsigned int *upload_data_size, void **ptr)
+	  size_t *upload_data_size, void **ptr)
 {
   static int aptr;
   struct MHD_Response *response;
